@@ -4,7 +4,7 @@ use strict;
 use warnings;
 use version;
 
-our $VERSION = version->declare('v4.24.0');
+our $VERSION = version->declare('v4.25.0');
 our $PATCHLV = 0;
 sub version { return $VERSION.($PATCHLV > 0 ? 'p'.$PATCHLV : '') }
 sub sysname { 'bouncehammer' }
@@ -56,8 +56,7 @@ sub make {
     if( $input eq 'email' ) {
         # Path to mailbox or Maildir/, or STDIN: 'input' => 'email'
         require Sisimai::Mail;
-        my $mail = Sisimai::Mail->new($argv0);
-        return undef unless $mail;
+        my $mail = Sisimai::Mail->new($argv0) || return undef;
 
         while( my $r = $mail->read ) {
             # Read and parse each mail file
@@ -67,8 +66,7 @@ sub make {
                 'input' => 'email',
                 'field' => $field,
             };
-            my $mesg = Sisimai::Message->new(%$methodargv);
-            next unless defined $mesg;
+            next unless my $mesg = Sisimai::Message->new(%$methodargv);
 
             my $data = Sisimai::Data->make('data' => $mesg, %$delivered1);
             push @$bouncedata, @$data if scalar @$data;
@@ -76,22 +74,21 @@ sub make {
     } elsif( $input eq 'json' ) {
         # Decoded JSON object: 'input' => 'json'
         my $type = ref $argv0;
-        my $list = [];
+        my @list;
 
         if( $type eq 'ARRAY' ) {
             # [ {...}, {...}, ... ]
             for my $e ( @$argv0 ) {
                 next unless ref $e eq 'HASH';
-                push @$list, $e;
+                push @list, $e;
             }
         } else {
-            push @$list, $argv0;
+            push @list, $argv0;
         }
 
-        while( my $e = shift @$list ) {
+        for my $e ( @list ) {
             $methodargv = { 'data' => $e, 'hook' => $hookmethod, 'input' => 'json' };
-            my $mesg = Sisimai::Message->new(%$methodargv);
-            next unless defined $mesg;
+            next unless my $mesg = Sisimai::Message->new(%$methodargv);
 
             my $data = Sisimai::Data->make('data' => $mesg, %$delivered1);
             push @$bouncedata, @$data if scalar @$data;
@@ -136,13 +133,11 @@ sub engine {
     # Parser engine list (MTA modules)
     # @return   [Hash]     Parser engine table
     my $class = shift;
-    my $names = [qw|Bite::Email Bite::JSON ARF RFC3464 RFC3834|];
     my $table = {};
-    my $loads = '';
 
-    while( my $e = shift @$names ) {
+    for my $e ('Bite::Email', 'Bite::JSON', 'ARF', 'RFC3464', 'RFC3834') {
         my $r = 'Sisimai::'.$e;
-        ($loads = $r) =~ s|::|/|g; 
+        (my $loads = $r) =~ s|::|/|g; 
         require $loads.'.pm';
 
         if( $e eq 'Bite::Email' || $e eq 'Bite::JSON' ) {
@@ -168,17 +163,14 @@ sub reason {
     my $class = shift;
     my $table = {};
 
-    require Sisimai::Reason;
-    my $names = Sisimai::Reason->index;
-    my $loads = '';
-
     # These reasons are not included in the results of Sisimai::Reason->index
-    push @$names, (qw|Delivered Feedback Undefined Vacation|);
+    require Sisimai::Reason;
+    my @names = (@{ Sisimai::Reason->index }, qw|Delivered Feedback Undefined Vacation|);
 
-    while( my $e = shift @$names ) {
+    for my $e ( @names ) {
         # Call ->description() method of Sisimai::Reason::*
         my $r = 'Sisimai::Reason::'.$e;
-        ($loads = $r) =~ s|::|/|g; 
+        (my $loads = $r) =~ s|::|/|g; 
         require $loads.'.pm';
         $table->{ $e } = $r->description;
     }
@@ -370,7 +362,7 @@ azumakuniyuki
 
 =head1 COPYRIGHT
 
-Copyright (C) 2014-2018 azumakuniyuki, All rights reserved.
+Copyright (C) 2014-2019 azumakuniyuki, All rights reserved.
 
 =head1 LICENSE
 
