@@ -38,7 +38,6 @@ sub scan {
     return undef unless index($mhead->{'subject'}, 'Undeliverable message') == 0;
 
     my $dscontents = [__PACKAGE__->DELIVERYSTATUS];
-    my @hasdivided = split("\n", $$mbody);
     my $rfc822part = '';    # (String) message/rfc822-headers part
     my $rfc822list = [];    # (Array) Each line in message/rfc822 part string
     my $blanklines = 0;     # (Integer) The number of blank lines
@@ -49,7 +48,7 @@ sub scan {
     my $encodedmsg = '';
     my $v = undef;
 
-    for my $e ( @hasdivided ) {
+    for my $e ( split("\n", $$mbody) ) {
         # Read each line between the start of the message and the start of rfc822 part.
         unless( $readcursor ) {
             # Beginning of the bounce message or delivery status part
@@ -74,16 +73,15 @@ sub scan {
         }
 
         if( $readcursor & $Indicators->{'message-rfc822'} ) {
-            # After "message/rfc822"
+            # Inside of the original message part
             unless( length $e ) {
-                $blanklines++;
-                last if $blanklines > 1;
+                last if ++$blanklines > 1;
                 next;
             }
             push @$rfc822list, $e;
 
         } else {
-            # Before "message/rfc822"
+            # Error message part
             next unless $readcursor & $Indicators->{'deliverystatus'};
 
             # ------- Failure Reasons  --------
@@ -126,17 +124,16 @@ sub scan {
                     $v->{'diagnosis'} .= $e;
                 }
             }
-        } # End of if: rfc822
+        } # End of error message part
     }
 
     unless( $recipients ) {
         # Fallback: Get the recpient address from RFC822 part
         for my $e ( @$rfc822list ) {
-            if( $e =~ /^To:[ ]*(.+)$/ ) {
-                $v->{'recipient'} = Sisimai::Address->s3s4($1);
-                $recipients++ if $v->{'recipient'};
-                last;
-            }
+            next unless $e =~ /^To:[ ]*(.+)$/;
+            $v->{'recipient'} = Sisimai::Address->s3s4($1);
+            $recipients++ if $v->{'recipient'};
+            last;
         }
     }
     return undef unless $recipients;
@@ -150,9 +147,7 @@ sub scan {
             # Check each regular expression of Notes error messages
             next unless grep { index($e->{'diagnosis'}, $_) > -1 } @{ $MessagesOf->{ $r } };
             $e->{'reason'} = $r;
-
-            my $pseudostatus = Sisimai::SMTP::Status->code($r);
-            $e->{'status'} = $pseudostatus if $pseudostatus;
+            $e->{'status'} = Sisimai::SMTP::Status->code($r) || '';
             last;
         }
     }
