@@ -48,7 +48,7 @@ my $StatusList = {
     qr/\A5[.]7[.]7[0-4]\d\z/ => 'toomanyconn',
 };
 my $ReCommands = {
-    'RCPT' => qr/unknown recipient or mailbox unavailable ->.+[<].+[@].+[>]/,
+    'RCPT' => qr/unknown recipient or mailbox unavailable ->.+[<]?.+[@].+[.][a-zA-Z]+[>]?/,
 };
 
 sub headerlist  { 
@@ -60,13 +60,14 @@ sub headerlist  {
     # X-MS-Exchange-CrossTenant-FromEntityHeader: Hosted
     # X-MS-Exchange-Transport-CrossTenantHeadersStamped: ...
     return [
-        'X-MS-Exchange-Message-Is-Ndr',
-        'X-Microsoft-Antispam-PRVS',
-        'X-Exchange-Antispam-Report-Test',
-        'X-Exchange-Antispam-Report-CFA-Test',
-        'X-MS-Exchange-CrossTenant-OriginalArrivalTime',
-        'X-MS-Exchange-CrossTenant-FromEntityHeader',
-        'X-MS-Exchange-Transport-CrossTenantHeadersStamped',
+        'content-language',
+        'x-ms-exchange-message-is-ndr',
+        'x-microsoft-antispam-prvs',
+        'x-exchange-antispam-report-test',
+        'x-exchange-antispam-report-cfa-test',
+        'x-ms-exchange-crosstenant-originalarrivaltime',
+        'x-ms-exchange-crosstenant-fromentityheader',
+        'x-ms-exchange-transport-crosstenantheadersstamped',
     ]
 }
 sub description { 'Microsoft Office 365: http://office.microsoft.com/' }
@@ -104,19 +105,20 @@ sub scan {
     }
     return undef if $match < 2;
 
+    require Sisimai::RFC1894;
+    my $fieldtable = Sisimai::RFC1894->FIELDTABLE;
+    my $permessage = {};    # (Hash) Store values of each Per-Message field
+
     my $dscontents = [__PACKAGE__->DELIVERYSTATUS];
-    my @hasdivided = split("\n", $$mbody);
     my $rfc822part = '';    # (String) message/rfc822-headers part
     my $rfc822list = [];    # (Array) Each line in message/rfc822 part string
     my $blanklines = 0;     # (Integer) The number of blank lines
     my $readcursor = 0;     # (Integer) Points the current cursor position
     my $recipients = 0;     # (Integer) The number of 'Final-Recipient' header
-    my $connheader = {};
     my $endoferror = 0;     # (Integer) Flag for the end of error messages
-    my $htmlbegins = 0;     # (Integer) Flag for HTML part
     my $v = undef;
 
-    for my $e ( @hasdivided ) {
+    for my $e ( split("\n", $$mbody) ) {
         # Read each line between the start of the message and the start of rfc822 part.
         unless( $readcursor ) {
             # Beginning of the bounce message or delivery status part
@@ -135,16 +137,15 @@ sub scan {
         }
 
         if( $readcursor & $Indicators->{'message-rfc822'} ) {
-            # After "message/rfc822"
+            # Inside of the original message part
             unless( length $e ) {
-                $blanklines++;
-                last if $blanklines > 1;
+                last if ++$blanklines > 1;
                 next;
             }
             push @$rfc822list, $e;
 
         } else {
-            # Before "message/rfc822"
+            # Error message part
             next unless $readcursor & $Indicators->{'deliverystatus'};
             next unless length $e;
 
@@ -156,7 +157,7 @@ sub scan {
 
             if( $e =~ /\A.+[@].+[<]mailto:(.+[@].+)[>]\z/ ) {
                 # kijitora@example.com<mailto:kijitora@example.com>
-                if( length $v->{'recipient'} ) {
+                if( $v->{'recipient'} ) {
                     # There are multiple recipient addresses in the message body.
                     push @$dscontents, __PACKAGE__->DELIVERYSTATUS;
                     $v = $dscontents->[-1];
@@ -166,41 +167,20 @@ sub scan {
 
             } elsif( $e =~ /\AGenerating server: (.+)\z/ ) {
                 # Generating server: FFFFFFFFFFFF.e0.prod.outlook.com
-                $connheader->{'lhost'} = lc $1;
+                $permessage->{'lhost'} = lc $1;
 
             } else {
                 if( $endoferror ) {
                     # After "Original message headers:"
-                    if( $htmlbegins ) {
-                        # <html> .. </html>
-                        $htmlbegins = 0 if index($e, '</html>') == 0;
-                        next;
-                    }
+                    next unless my $f = Sisimai::RFC1894->match($e);
+                    next unless my $o = Sisimai::RFC1894->field($e);
+                    next unless exists $fieldtable->{ $o->[0] };
+                    next if $o->[0] =~ /\A(?:diagnostic-code|final-recipient)\z/;
+                    $v->{ $fieldtable->{ $o->[0] } } = $o->[2];
 
-                    if( $e =~ /\AAction:[ ]*(.+)\z/ ) {
-                        # Action: failed
-                        $v->{'action'} = lc $1;
+                    next unless $f == 1;
+                    $permessage->{ $fieldtable->{ $o->[0] } } = $o->[2];
 
-                    } elsif( $e =~ /\AStatus:[ ]*(\d[.]\d+[.]\d+)/ ) {
-                        # Status:5.2.0
-                        $v->{'status'} = $1;
-
-                    } elsif( $e =~ /\AReporting-MTA:[ ]*(?:DNS|dns);[ ]*(.+)\z/ ) {
-                        # Reporting-MTA: dns;BLU004-OMC3S13.hotmail.example.com
-                        $connheader->{'lhost'} = lc $1;
-
-                    } elsif( $e =~ /\AReceived-From-MTA:[ ]*(?:DNS|dns);[ ]*(.+)\z/ ) {
-                        # Reporting-MTA: dns;BLU004-OMC3S13.hotmail.example.com
-                        $connheader->{'rhost'} = lc $1;
-
-                    } elsif( $e =~ /\AArrival-Date:[ ]*(.+)\z/ ) {
-                        # Arrival-Date: Wed, 29 Apr 2009 16:03:18 +0900
-                        next if length $connheader->{'date'};
-                        $connheader->{'date'} = $1;
-
-                    } else {
-                        $htmlbegins = 1 if index($e, '<html>') == 0;
-                    }
                 } else {
                     if( $e eq $StartingOf->{'error'}->[0] ) {
                         # Diagnostic information for administrators:
@@ -220,23 +200,19 @@ sub scan {
                     }
                 }
             }
-        } # End of if: rfc822
+        } # End of error message part
     }
     return undef unless $recipients;
 
-    require Sisimai::String;
-    require Sisimai::SMTP::Status;
     for my $e ( @$dscontents ) {
         # Set default values if each value is empty.
-        map { $e->{ $_ } ||= $connheader->{ $_ } || '' } keys %$connheader;
-
+        map { $e->{ $_ } ||= $permessage->{ $_ } || '' } keys %$permessage;
         $e->{'agent'}     = __PACKAGE__->smtpagent;
         $e->{'diagnosis'} = Sisimai::String->sweep($e->{'diagnosis'});
 
-        if( length($e->{'status'}) == 0 || substr($e->{'status'}, -4, 4) eq '.0.0' ) {
+        if( ! $e->{'status'} || substr($e->{'status'}, -4, 4) eq '.0.0' ) {
             # There is no value of Status header or the value is 5.0.0, 4.0.0
-            my $r = Sisimai::SMTP::Status->find($e->{'diagnosis'});
-            $e->{'status'} = $r if length $r;
+            $e->{'status'} = Sisimai::SMTP::Status->find($e->{'diagnosis'}) || $e->{'status'};
         }
 
         for my $p ( keys %$ReCommands ) {
@@ -246,9 +222,8 @@ sub scan {
             last;
         }
 
-        next unless $e->{'status'};
-
         # Find the error code from $StatusList
+        next unless $e->{'status'};
         for my $f ( keys %$StatusList ) {
             # Try to match with each key as a regular expression
             next unless $e->{'status'} =~ $f;
@@ -303,7 +278,7 @@ azumakuniyuki
 
 =head1 COPYRIGHT
 
-Copyright (C) 2016-2018 azumakuniyuki, All rights reserved.
+Copyright (C) 2016-2019 azumakuniyuki, All rights reserved.
 
 =head1 LICENSE
 
